<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\Registry\Registry;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * VP Advanced Custom Field Utility Helper
 */
class VMCustomAdvancedUtility
{
    public static $encoding = 'base';

    public static $encoding_suffix = '64';

    public static function encode($string)
    {
        $function = strtolower(self::$encoding . self::$encoding_suffix . '_encode');

        if (function_exists($function)) {
            return call_user_func($function, $string);
        }

        return false;
    }

    public static function decode($string)
    {
        $function = strtolower(self::$encoding . self::$encoding_suffix . '_decode');

        if (function_exists($function)) {
            return call_user_func($function, $string);
        }

        return false;
    }

    public static function getEncodeFilter()
    {
        return strtoupper(self::$encoding . self::$encoding_suffix);
    }

    public static function jsonResponse($output = array())
    {
        $app     = Factory::getApplication();
        $obLevel = ob_get_level();

        if ($obLevel) {
            while ($obLevel > 0) {
                ob_end_clean();
                $obLevel--;
            }
        } elseif (ob_get_contents()) {
            ob_clean();
        }

        header('Content-type: application/json');
        header('Cache-Control: public,max-age=1,must-revalidate');
        header('Expires: ' . gmdate('D, d M Y H:i:s', ($_SERVER['REQUEST_TIME'] + 1)) . ' GMT');
        header('Last-modified: ' . gmdate('D, d M Y H:i:s', $_SERVER['REQUEST_TIME']) . ' GMT');

        if (function_exists('header_remove')) {
            header_remove('Pragma');
        }

        $response = new Registry();
        $response->loadArray($output);

        echo (string) $response;

        flush();
        $app->close();
    }
}
