<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;

if (!trait_exists('VmcustomadvancedModelHistoryTrait')) {
    require __DIR__ . '/historytrait.php';
}
// phpcs:enable PSR1.Files.SideEffects

/**
 * VMCustomAdvanced model for set.
 */
class VMCustomAdvancedModelSet extends JModelAdmin
{
    use VmcustomadvancedModelHistoryTrait;

    /**
     * @var   string  The prefix to use with controller messages.
     */
    protected $text_prefix = 'COM_VMCUSTOMADVANCED';

    /**
     * @var   string  Alias to manage history control
     */
    public $typeAlias = 'com_vmcustomadvanced.set';

    /**
     * @var null  Item data
     */
    protected $item = null;

    /**
     * Returns a reference to the a Table object, always creating it.
     *
     * @param   string  $type    The table type to instantiate
     * @param   string  $prefix  A prefix for the table class name. Optional.
     * @param   array   $config  Configuration array for model. Optional.
     *
     * @return  JTable    A database object
     */
    public function getTable($type = 'Set', $prefix = 'VMCustomAdvancedTable', $config = [])
    {
        return JTable::getInstance($type, $prefix, $config);
    }

    /**
     * Method to get the record form.
     *
     * @param   array    $data      An optional array of data for the form to interogate.
     * @param   boolean  $loadData  True if the form is to load its own data (default case), false if not.
     *
     * @return  JForm  A JForm object on success, false on failure
     */
    public function getForm($data = [], $loadData = true)
    {
        // Get the form.
        $form = $this->loadForm('com_vmcustomadvanced.set', 'set', ['control' => 'jform', 'load_data' => $loadData]);

        if (empty($form)) {
            return false;
        }

        return $form;
    }

    /**
     * Method to allow derived classes to preprocess the form.
     *
     * @param   JForm   $form   A JForm object.
     * @param   mixed   $data   The data expected for the form.
     * @param   string  $group  The name of the plugin group to import (defaults to "content").
     *
     * @return  void
     *
     * @see     JFormField
     * @throws  Exception if there is an error in the form event.
     */
    protected function preprocessForm($form, $data, $group = 'content')
    {
        if (!empty($form)) {
            if (version_compare(JVERSION, '4.0.0', 'ge')) {
                $form->setFieldAttribute('state', 'class', '');
                $form->setFieldAttribute('state', 'layout', 'joomla.form.field.radio.switcher');

                $form->setFieldAttribute('is_button', 'class', '');
                $form->setFieldAttribute('is_button', 'layout', 'joomla.form.field.radio.switcher');

                $form->setFieldAttribute('filter_support', 'class', '');
                $form->setFieldAttribute('filter_support', 'layout', 'joomla.form.field.radio.switcher');

                $form->setFieldAttribute('translatable_filter_options', 'class', '');
                $form->setFieldAttribute('translatable_filter_options', 'layout', 'joomla.form.field.radio.switcher');
            }
        }

        parent::preprocessForm($form, $data, $group);
    }

    /**
     * Method to get the data that should be injected in the form.
     *
     * @return   mixed  The data for the form.
     */
    protected function loadFormData()
    {
        /** @var \Joomla\CMS\Application\CMSApplication */
        $app = Factory::getApplication();

        // Check the session for previously entered form data.
        $data = $app->getUserState('com_vmcustomadvanced.edit.set.data', []);

        if (empty($data)) {
            if ($this->item === null) {
                $this->item = $this->getItem();
            }

            /** @var \VMCustomAdvancedTableSet $data */
            $data = $this->item;

            // Support for multiple or not foreign key field: type
            $array = array();

            foreach ((array) $data->type as $value) {
                if (!is_array($value)) {
                    $array[] = $value;
                }
            }

            if (!empty($array)) {
                $data->type = $array;
            }
        }

        return $data;
    }

    /**
     * Method to get a single record.
     *
     * @param   integer  $pk  The id of the primary key.
     *
     * @return  mixed    Object on success, false on failure.
     */
    public function getItem($pk = null)
    {
        if ($item = parent::getItem($pk)) {
            // Do any procesing on fields here if needed
        }

        return $item;
    }

    /**
     * Method to save the form data.
     *
     * @param   array  $data  The form data.
     *
     * @return  boolean  True on success, False on error.
     */
    public function save($data)
    {
        $result = parent::save($data);

        if ($result) {
            if (VMCustomAdvancedHelper::buildLanguageTranslation('sets') === false) {
                $this->setError('Failed to create language translation files.');

                return false;
            }
        }

        return $result;
    }
}
