<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * option Table class
 */
class VMCustomAdvancedTableOrder_items extends JTable
{
    /**
     * Constructor
     *
     * @param   Joomla\Database\DatabaseDriver  &$db  A database connector object
     */
    public function __construct(&$db)
    {
        parent::__construct('#__vmcustomadvanced_order_items', 'id', $db);
    }

    /**
     * Overloaded bind function to pre-process the params.
     *
     * @param   array  $array   Named array
     * @param   mixed  $ignore  Optional array or list of parameters to ignore
     *
     * @return  null|string  null is operation was satisfactory, otherwise returns an error
     *
     * @see     JTable:bind
     */
    public function bind($array, $ignore = '')
    {
        $app   = Factory::getApplication();
        $input = $app->input;
        $task  = $input->input->get('task');

        return parent::bind($array, $ignore);
    }

    /**
     * Overloaded check function
     *
     * @return bool
     */
    public function check()
    {
        return parent::check();
    }

    /**
     * Delete a record by id
     *
     * @param   mixed  $pk  Primary key value to delete. Optional
     *
     * @return bool
     */
    public function delete($pk = null)
    {
        $this->load($pk);
        $result = parent::delete($pk);

        return $result;
    }

    /**
    * Method to store a node in the database table.
    *
    * @param   boolean  $updateNulls  True to update null values as well.
    *
    * @return  boolean  True on success.
    *
    * @link    http://docs.joomla.org/JTableNested/store
    */
    public function store($updateNulls = false)
    {
        $date = Factory::getDate();
        $user = Factory::getUser();

        if ($this->id) {
            // Existing item
            $this->modified_on = $date->toSql();
            $this->modified_by   = $user->id;
        } else {
            if (!(int) $this->created_on) {
                $this->created_on = $date->toSql();
            }

            if (empty($this->created_by)) {
                $this->created_by = $user->id;
            }
        }

        return parent::store($updateNulls);
    }
}
