<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Router\Route;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

HTMLHelper::addIncludePath(JPATH_COMPONENT . '/helpers/html');

if (version_compare(JVERSION, '4.0.0', 'ge')) {
    $wa = $this->document->getWebAssetManager();
    $wa->useScript('keepalive')
        ->useScript('form.validate');
} else {
    HTMLHelper::_('behavior.tooltip');
    HTMLHelper::_('behavior.formvalidator');
    HTMLHelper::_('formbehavior.chosen', 'select');
    HTMLHelper::_('behavior.keepalive');
}

$document = JFactory::getDocument();

$document->addScriptDeclaration("
    var saveState = function() {
        var form = jQuery('#option-form');
        form.find('input[name=\"task\"]').val('option.saveState');
        form.submit();
    };
    
    Joomla.submitbutton = function(task) {
        if (task == 'option.cancel') {
            Joomla.submitform(task, document.getElementById('option-form'));
        } else {
            if (task != 'option.cancel' && document.formvalidator.isValid(document.getElementById('option-form'))) {
                Joomla.submitform(task, document.getElementById('option-form'));
            } else {
                alert('" . $this->escape(Text::_('JGLOBAL_VALIDATION_FORM_FAILED')) . "');
            }
        }
    };
");
?>
<form action="<?php echo Route::_('index.php?option=com_vmcustomadvanced&layout=edit&id=' . (int) $this->item->id); ?>" method="post" name="adminForm" id="option-form" class="form-validate">
    <div class="form-horizontal">
        <?php echo $this->form->renderField('title'); ?>
        <?php echo $this->form->renderField('alias'); ?>
        <?php echo $this->form->renderField('set_id'); ?>
        <?php echo $this->form->renderField('color'); ?>
        <?php echo $this->form->renderField('image'); ?>
        <?php echo $this->form->renderField('tick_color', 'params'); ?>
        <?php echo $this->form->renderField('state'); ?>
        <?php echo $this->form->renderField('created_by'); ?>
        <?php echo $this->form->renderField('created_on'); ?>
        <?php if (!empty($this->item->created_on) && $this->item->created_on != $this->item->modified_on) : ?>
            <?php echo $this->form->renderField('modified_by'); ?>
            <?php echo $this->form->renderField('modified_on'); ?>
        <?php endif; ?>
        <?php if ($this->state->params->get('save_history', 1)) : ?>
            <?php echo $this->form->renderField('version_note'); ?>
        <?php endif; ?>
    </div>
    
    <?php echo $this->form->renderField('id'); ?>
    <?php echo $this->form->renderField('ordering'); ?>
    <?php echo $this->form->renderField('checked_out'); ?>
    <?php echo $this->form->renderField('checked_out_time'); ?>
    <input type="hidden" name="task" value=""/>
    <?php echo HTMLHelper::_('form.token'); ?>
</form>
