<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Layout\LayoutHelper;
use Joomla\CMS\Router\Route;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

HTMLHelper::addIncludePath(JPATH_COMPONENT . '/helpers/');

if (!version_compare(JVERSION, '4.0.0', 'ge')) {
    HTMLHelper::_('bootstrap.tooltip');
    HTMLHelper::_('behavior.multiselect');
    HTMLHelper::_('formbehavior.chosen', 'select');
}

$document = Factory::getDocument();

$document->addStyleDeclaration("
    .option-display {
        display: inline-block;
        vertical-align: middle;
        line-height: 1;
    }
    .option-color {
        display: inline-block;
        border-radius: 3px;
        width: 18px;
        height: 18px;
        border: 1px solid #ccc;
        margin-right: 5px;
        -webkit-box-shadow: inset 0 9px 0 rgba(255, 255, 255, .1);
        -moz-box-shadow: inset 0 9px 0 rgba(255, 255, 255, .1);
        box-shadow: inset 0 9px 0 rgba(255, 255, 255, .1);
    }
    .option-image {
        display: inline-block;
        border-radius: 3px;
        width: 18px;
        height: auto;
        max-height: 18px;
        border: 1px solid #ccc;
        margin-right: 5px;
    }
");

if (!version_compare(JVERSION, '4.0.0', 'ge')) {
    $document->addScriptDeclaration("
        var toggleField = function (id, task, field) {
            var f = document.adminForm, i = 0, cbx, cb = f[ id ];
            if (!cb) return false;

            while (true) {
                cbx = f[ 'cb' + i ];
                if (!cbx) break;
                cbx.checked = false;
                i++;
            }

            var inputField   = document.createElement('input');

            inputField.type  = 'hidden';
            inputField.name  = 'field';
            inputField.value = field;
            f.appendChild(inputField);

            cb.checked = true;
            f.boxchecked.value = 1;
            window.submitform(task);

            return false;
        };
    ");
}

$user      = Factory::getUser();
$userId    = $user->get('id');
$listOrder = $this->state->get('list.ordering');
$listDirn  = $this->state->get('list.direction');
$canOrder  = $user->authorise('core.edit.state', 'com_vmcustomadvanced');
$saveOrder = $listOrder == 'a.ordering';

if ($saveOrder) {
    $saveOrderingUrl = 'index.php?option=com_vmcustomadvanced&task=options.saveOrderAjax&tmpl=component';
    HTMLHelper::_('sortablelist.sortable', 'optionList', 'adminForm', strtolower($listDirn), $saveOrderingUrl);
}
?>
<form action="<?php echo Route::_('index.php?option=com_vmcustomadvanced&view=options'); ?>" method="post" name="adminForm" id="adminForm">
    <?php if (!empty($this->sidebar)) : ?>
        <div id="j-sidebar-container" class="span2">
            <?php echo $this->sidebar; ?>
        </div>
        <div id="j-main-container" class="span10">
    <?php else : ?>
        <div id="j-main-container">
    <?php endif; ?>
            <?php echo LayoutHelper::render('joomla.searchtools.default', array('view' => $this)); ?>
            <div class="clearfix"></div>
            <table class="table table-striped" id="optionList">
                <thead>
                    <tr>
                        <th width="1%" class="nowrap text-nowrap center text-center hidden-phone d-none d-md-table-cell">
                            <?php echo HTMLHelper::_('searchtools.sort', '', 'a.ordering', $listDirn, $listOrder, null, 'asc', 'JGRID_HEADING_ORDERING', 'icon-menu-2'); ?>
                        </th>
                        <th width="1%" class="hidden-phone d-none d-md-table-cell">
                            <input type="checkbox" name="checkall-toggle" value="" title="<?php echo Text::_('JGLOBAL_CHECK_ALL'); ?>" onclick="Joomla.checkAll(this)" />
                        </th>
                        <th width="1%" class="nowrap text-nowrap center text-center">
                            <?php echo HTMLHelper::_('searchtools.sort', 'JSTATUS', 'a.state', $listDirn, $listOrder); ?>
                        </th>
                        <th class="left">
                            <?php echo HTMLHelper::_('searchtools.sort', 'COM_VMCUSTOMADVANCED_OPTIONS_TITLE', 'a.title', $listDirn, $listOrder); ?>
                        </th>
                        <th class="left">
                            <?php echo HTMLHelper::_('searchtools.sort', 'COM_VMCUSTOMADVANCED_TITLE_SET', 's.title', $listDirn, $listOrder); ?>
                        </th>
                        <th class="left nowrap text-nowrap d-none d-md-table-cell" width="1%">
                            <?php echo HTMLHelper::_('searchtools.sort', 'COM_VMCUSTOMADVANCED_OPTIONS_CREATED_BY', 'a.created_by', $listDirn, $listOrder); ?>
                        </th>
                        <th class="left nowrap text-nowrap d-none d-md-table-cell" width="1%">
                            <?php echo HTMLHelper::_('searchtools.sort', 'COM_VMCUSTOMADVANCED_OPTIONS_CREATED_ON', 'a.created_on', $listDirn, $listOrder); ?>
                        </th>
                        <th class="left d-none d-md-table-cell" width="1%">
                            <?php echo HTMLHelper::_('searchtools.sort', 'COM_VMCUSTOMADVANCED_OPTIONS_ID', 'a.id', $listDirn, $listOrder); ?>
                        </th>
                    </tr>
                </thead>
                <tfoot>
                    <tr>
                        <td colspan="8">
                            <?php echo $this->pagination->getListFooter(); ?>
                        </td>
                    </tr>
                </tfoot>
                <tbody>
                    <?php foreach ($this->items as $i => $item) :
                        $ordering   = ($listOrder == 'a.ordering');
                        $canCreate  = $user->authorise('core.create', 'com_vmcustomadvanced');
                        $canEdit    = $user->authorise('core.edit', 'com_vmcustomadvanced');
                        $canCheckin = $user->authorise('core.manage', 'com_vmcustomadvanced');
                        $canChange  = $user->authorise('core.edit.state', 'com_vmcustomadvanced');
                        ?>
                        <tr class="row<?php echo $i % 2; ?>">
                            <td class="order nowrap text-nowrap center text-center hidden-phone d-none d-md-table-cell">
                                <?php if ($canChange) :
                                    $disableClassName = '';
                                    $disabledLabel    = '';
                                    if (!$saveOrder) {
                                        $disabledLabel    = Text::_('JORDERINGDISABLED');
                                        $disableClassName = 'inactive tip-top';
                                    } ?>
                                    <span class="sortable-handler hasTooltip <?php echo $disableClassName ?>" title="<?php echo $disabledLabel ?>">
                                        <i class="icon-menu"></i>
                                    </span>
                                    <input type="text" style="display:none" name="order[]" size="5" value="<?php echo $item->ordering; ?>" class="width-20 text-area-order "/>
                                <?php else : ?>
                                    <span class="sortable-handler inactive"><i class="icon-menu"></i></span>
                                <?php endif; ?>
                            </td>
                            <td class="hidden-phone d-none d-md-table-cell">
                                <?php echo HTMLHelper::_('grid.id', $i, $item->id); ?>
                            </td>
                            <td class="center text-center">
                                <?php echo HTMLHelper::_('jgrid.published', $item->state, $i, 'options.', $canChange, 'cb'); ?>
                            </td>
                            <td>
                                <?php if ($item->option_type) : ?>
                                    <div class="option-display">
                                        <?php if ($item->option_type == 1) : ?>
                                            <span class="option-color" style="background-color: <?php echo $item->color; ?>"></span>
                                        <?php elseif ($item->option_type == 2) : ?>
                                            <img class="option-image" src="<?php echo JUri::root() . $item->image; ?>" alt="<?php echo $this->escape($item->title); ?>" />
                                        <?php endif; ?>
                                    </div>
                                <?php endif; ?>
                                <?php if (isset($item->checked_out) && $item->checked_out && ($canEdit || $canChange)) : ?>
                                    <?php echo HTMLHelper::_('jgrid.checkedout', $i, $item->uEditor, $item->checked_out_time, 'options.', $canCheckin); ?>
                                <?php endif; ?>
                                <?php if ($canEdit) : ?>
                                    <a href="<?php echo JRoute::_('index.php?option=com_vmcustomadvanced&task=option.edit&id=' . (int) $item->id); ?>"><?php echo $this->escape($item->title); ?></a>
                                <?php else : ?>
                                    <?php echo $this->escape($item->title); ?>
                                <?php endif; ?>
                                <span class="small break-word d-none d-md-inline-block ms-1">
                                    (<span><?php echo Text::_('COM_VMCUSTOMADVANCED_FORM_LBL_ALIAS'); ?></span>: <?php echo $item->alias; ?>)
                                </span>
                            </td>
                            <td class="small">
                                <?php if ($canEdit) : ?>
                                    <a href="<?php echo Route::_('index.php?option=com_vmcustomadvanced&task=set.edit&id=' . (int) $item->set_id); ?>"><?php echo $this->escape($item->set_title); ?></a>
                                <?php else : ?>
                                    <?php echo $this->escape($item->set_title); ?>
                                <?php endif; ?>
                            </td>
                            <td class="nowrap text-nowrap small d-none d-md-table-cell">
                                <?php echo $item->created_by; ?>
                            </td>
                            <td class="nowrap text-nowrap small d-none d-md-table-cell">
                                <?php echo $item->created_on; ?>
                            </td>
                            <td class="small d-none d-md-table-cell">
                                <?php echo $item->id; ?>
                            </td>
                        </tr>
                    <?php endforeach; ?>
                </tbody>
            </table>

            <input type="hidden" name="task" value=""/>
            <input type="hidden" name="boxchecked" value="0"/>
            <input type="hidden" name="list[fullorder]" value="<?php echo $listOrder; ?> <?php echo $listDirn; ?>"/>
            <?php echo HTMLHelper::_('form.token'); ?>
        </div>
</form>