<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Language\Text;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

/**
 * View class for a list of sets.
 */
class VMCustomAdvancedViewSets extends JViewLegacy
{
    protected $items;

    protected $pagination;

    protected $state;

    /**
     * Display the view
     *
     * @param   string  $tpl  Template name
     *
     * @return void
     *
     * @throws Exception
     */
    public function display($tpl = null)
    {
        $this->state         = $this->get('State');
        $this->items         = $this->get('Items');
        $this->pagination    = $this->get('Pagination');
        $this->filterForm    = $this->get('FilterForm');
        $this->activeFilters = $this->get('ActiveFilters');

        // Check for errors.
        if (count($errors = $this->get('Errors'))) {
            throw new Exception(implode("\n", $errors));
        }

        VmcustomadvancedHelper::addSubmenu('sets');

        $this->addToolbar();

        if (!version_compare(JVERSION, '4.0.0', 'ge')) {
            $this->sidebar = JHtmlSidebar::render();
        }

        parent::display($tpl);
    }

    /**
     * Add the page title and toolbar.
     *
     * @return void
     */
    protected function addToolbar()
    {
        $canDo = VMCustomAdvancedHelper::getActions();

        JToolBarHelper::title(Text::_('COM_VMCUSTOMADVANCED') . ': ' . Text::_('COM_VMCUSTOMADVANCED_TITLE_SETS'), 'database sets');

        if ($canDo->get('core.create')) {
            JToolBarHelper::addNew('set.add', 'JTOOLBAR_NEW');
        }

        if ($canDo->get('core.edit') && isset($this->items[0])) {
            JToolBarHelper::editList('set.edit', 'JTOOLBAR_EDIT');
        }

        if ($canDo->get('core.edit.state') && isset($this->items[0])) {
            JToolBarHelper::divider();
            JToolBarHelper::custom('sets.publish', 'publish.png', 'publish_f2.png', 'JTOOLBAR_PUBLISH', true);
            JToolBarHelper::custom('sets.unpublish', 'unpublish.png', 'unpublish_f2.png', 'JTOOLBAR_UNPUBLISH', true);
            JToolBarHelper::deleteList('COM_VMCUSTOMADVANCED_WARNING_BEFORE_DELETE', 'sets.delete', 'JTOOLBAR_DELETE');
            JToolBarHelper::custom('sets.checkin', 'checkin.png', 'checkin_f2.png', 'JTOOLBAR_CHECKIN', true);
        }

        if ($canDo->get('core.admin')) {
            JToolBarHelper::preferences('com_vmcustomadvanced');
        }

        // Set sidebar action - New in 3.0
        JHtmlSidebar::setAction('index.php?option=com_vmcustomadvanced&view=sets');
    }
}
