<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\Registry\Registry;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

class VMCustomAdvancedControllerField extends JControllerLegacy
{
    protected $errorMessage = '';

    public function getChildVariants()
    {
        $app               = Factory::getApplication();
        $productModel      = VmModel::getModel('product');
        $customfieldsModel = VmModel::getModel('Customfields');
        $product_id        = $this->input->getInt('product_id', 0);
        $custom_id         = $this->input->getInt('custom_id', 0);
        $quantity          = $this->input->getInt('quantity', 1);
        $field             = null;

        if ($product_id <= 0) {
            return $this->returnFalse('Invalid product id.');
        }

        if ($custom_id <= 0) {
            return $this->returnFalse('Invalid custom id.');
        }

        $product = $productModel->getProduct($product_id, true, true, true, $quantity);

        if (empty($product)) {
            return $this->returnFalse('Product not found.');
        }

        if (empty($product->customfields)) {
            return $this->returnFalse('No custom fields found for the selected product.');
        }

        foreach ($product->customfields as $customfield) {
            if ($customfield->virtuemart_custom_id == $custom_id) {
                $field = $customfield;

                break;
            }
        }

        if (empty($field)) {
            return $this->returnFalse('No custom field found for the given custom id.');
        }

        JPluginHelper::importPlugin('vmcustom', 'advanced');

        $html = '';

        if (version_compare(JVERSION, '4.0.0', 'ge')) {
            $results = $app->triggerEvent('onDisplayProductSubVariants', [$this, $product, $field, &$html]);
        } else {
            $dispatcher = JEventDispatcher::getInstance();
            $results    = $dispatcher->trigger('onDisplayProductSubVariants', [$this, $product, $field, &$html]);
        }

        foreach ($results as $result) {
            if ($result === false) {
                return $this->returnFalse($this->getErrorMessage());
            }
        }

        return $this->returnTrue('', ['html' => $html]);
    }

    public function getErrorMessage()
    {
        return $this->errorMessage;
    }

    public function setErrorMessage(string $errorMessage)
    {
        $this->errorMessage = $errorMessage;
    }

    protected function returnTrue($message = '', $details = [])
    {
        $details = (array) $details;
        $output  = [];

        $output['result'] = true;
        $output['message'] = '';

        if (!empty($message)) {
            $output['message'] = $message;
        }

        if (count($details)) {
            $output = array_merge($output, $details);
        }

        return self::jsonResponse($output);
    }

    protected function returnFalse($message = '', $details = [])
    {
        $details = (array) $details;
        $output  = [];

        $output['result'] = false;
        $output['message'] = '';

        if (!empty($message)) {
            $output['message'] = $message;
        }

        if (count($details)) {
            $output = array_merge($output, $details);
        }

        return self::jsonResponse($output);
    }

    protected static function jsonResponse($output = [])
    {
        $app     = Factory::getApplication();
        $obLevel = ob_get_level();

        if ($obLevel) {
            while ($obLevel > 0) {
                ob_end_clean();
                $obLevel--;
            }
        } elseif (ob_get_contents()) {
            @ob_clean();
        }

        header('Content-type: application/json');
        header('Cache-Control: public,max-age=1,must-revalidate');
        header('Expires: ' . gmdate('D, d M Y H:i:s', ($_SERVER['REQUEST_TIME'] + 1)) . ' GMT');
        header('Last-modified: ' . gmdate('D, d M Y H:i:s', $_SERVER['REQUEST_TIME']) . ' GMT');

        if (function_exists('header_remove')) {
            header_remove('Pragma');
        }

        $response = new Registry();
        $response->loadArray($output);

        echo (string) $response;

        flush();
        $app->close();
    }
}
