<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;

include_once JPATH_SITE . '/components/com_vmcustomadvanced/define.php';

JLoader::import('joomla.filesystem.folder');
JLoader::import('joomla.filesystem.file');
// phpcs:enable PSR1.Files.SideEffects

/**
* VP Advanced Custom Field System Plugin
*
* @package   VP Advanced Custom Field
* @since     1.0
*/
class PlgSystemVMCustomAdvanced extends JPlugin
{
    protected $app;

    /**
     * Constructor.
     *
     * @param   object  &$subject  The object to observe
     * @param   array   $config    An optional associative array of configuration settings.
     */
    public function __construct(&$subject, $config)
    {
        parent::__construct($subject, $config);
    }

    /**
     * On content form preparation.
     *
     * @return  void
     */
    public function onContentPrepareForm($form, $data)
    {
        if (!($form instanceof JForm) || !($form instanceof \Joomla\CMS\Form\Form)) {
            return;
        }

        JForm::addFormPath(JPATH_ADMINISTRATOR . '/components/com_vmcustomadvanced/models/forms');
        JForm::addFieldPath(JPATH_ADMINISTRATOR . '/components/com_vmcustomadvanced/models/fields');

        $context = $form->getName();

        if ($context == 'com_config.component') {
            $input     = $this->app->input;
            $component = strtolower($input->getCmd('component', ''));

            if ($component === 'com_vmcustomadvanced') {
                if (self::hasExtension('pkg_vmcustomadvanced', 'package')) {
                    $form->loadFile('config', false);

                    if (version_compare(JVERSION, '4.0.0', 'ge')) {
                        $form->setFieldAttribute('save_history', 'class', '');
                        $form->setFieldAttribute('save_history', 'layout', 'joomla.form.field.radio.switcher');
                    }
                }
            }
        }
    }

    /**
     * Called on ajax get event.
     *
     * @return  boolean
     */
    public function onAjaxVmcustomadvanced()
    {
        if ($this->app->isClient('site') || Factory::getUser()->guest) {
            return;
        }

        // Register dlk helper class
        JLoader::register('VPComDlkHelper', JPATH_ADMINISTRATOR . '/components/com_vmcustomadvanced/models/fields/vpcomdlk/helper.php');

        // Get an instance of dlk helper
        $dlkHelper = VPComDlkHelper::getInstance();

        if ($this->app->input->getCmd('method') == 'validatedlk') {
            // Return validation result
            return $dlkHelper->validate();
        } elseif ($this->app->input->getCmd('method') == 'revalidatedlk') {
            // Return validation result
            return $dlkHelper->revalidate();
        }
    }

    /**
     * Called on live update installation.
     *
     * @return  boolean
     */
    public function onInstallerBeforePackageDownload(&$url, &$headers)
    {
        // Register dlk helper class
        JLoader::register('VPComDlkHelper', JPATH_ADMINISTRATOR . '/components/com_vmcustomadvanced/models/fields/vpcomdlk/helper.php');

        return VPComDlkHelper::getInstance()->addDlk($url, $headers);
    }

    /**
    * Method to check if an extension installed and enabled
    *
    * @param  string  $element  Name of the extension
    * @param  string  $type     Type of the extension like component, module, plugins etc
    *
    * @return boolean
    */
    /**
    * Method to check if an extension installed and enabled
    *
    * @param  string  $element  Name of the extension
    * @param  string  $type     Type of the extension like component, module, plugins etc
    *
    * @return boolean
    */
    private static function hasExtension($element, $type = 'component', $folder = null)
    {
        // Register component helper class
        JLoader::register('VMCustomAdvancedHelper', JPATH_ADMINISTRATOR . '/components/com_vmcustomadvanced/helpers/vmcustomadvanced.php');

        return VMCustomAdvancedHelper::hasExtension($element, $type, $folder);
    }
}
