<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\Registry\Registry;

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;

include_once JPATH_SITE . '/components/com_vmcustomadvanced/define.php';
// phpcs:enable PSR1.Files.SideEffects

class PlgVmCustomAdvanced extends vmCustomPlugin
{
    protected $app;

    protected $processedFields = [];

    protected $cartSelectedFields = [];

    protected $cartLayout = 'cart';

    protected static $sets = [];

    protected static $productFields = [];

    protected static $data = null;

    public function __construct(&$subject, $config)
    {
        parent::__construct($subject, $config);

        $this->setConfigParameterable('customfield_params', $this->getOptionVariables());

        $lang = Factory::getLanguage();

        $lang->load('com_vmcustomadvanced');
        $lang->load('com_vmcustomadvanced_sets');
        $lang->load('com_vmcustomadvanced_options');

        JLoader::register('PlgVmCustomAdvancedHelper', __DIR__ . '/helper.php');
    }

    public function onAfterDispatch()
    {
        if (!$this->app->isClient('administrator') || !$this->isHTML()) {
            return;
        }

        $input   = $this->app->input;
        $option  = strtolower($input->getCmd('option', ''));
        $view    = strtolower($input->getCmd('view', ''));
        $task    = strtolower($input->getCmd('task', ''));
        $version = PlgVmCustomAdvancedHelper::getVersion();

        if ($option === 'com_virtuemart' && in_array($view, ['custom', 'product'])) {
            HTMLHelper::_('jquery.framework');

            HTMLHelper::_('stylesheet', 'com_vmcustomadvanced/admin.css', ['version' => 'ver=' . $version, 'relative' => true]);
            HTMLHelper::_('script', 'com_vmcustomadvanced/admin.js', ['version' => 'ver=' . $version, 'relative' => true]);
        }
    }

    public function plgVmOnProductEdit($field, $product_id, &$row, &$retValue)
    {
        if ($field->custom_element != $this->_name) {
            return;
        }

        if (empty($field->field_sets)) {
            return;
        }

        $sets = $this->getSets($field->field_sets);

        if (empty($sets)) {
            return;
        }

        $formControl   = 'customfield_params[' . $row . ']';
        $formControlID = 'customfield_params_' . $row . '_';

        $retValue .= PlgVmCustomAdvancedHelper::renderLayout('product_be', ['field' => $field, 'sets' => $sets, 'formControl' => $formControl, 'formControlID' => $formControlID]);

        $row++;

        return true ;
    }

    public function plgVmOnDisplayProductFEVM3(&$product, &$field)
    {
        if ($field->custom_element != $this->_name) {
            return;
        }

        $hash = md5('virtuemart_product_id:' . $field->virtuemart_product_id . '.virtuemart_custom_id:' . $field->virtuemart_custom_id);

        if (isset($this->processedFields[$hash]) && $this->processedFields[$hash] != $field->virtuemart_customfield_id) {
            return;
        }

        if (!($fields = $this->getFullFieldsData($product, $field))) {
            return;
        }

        $max_level = count($fields->sets);
        $query_url = JUri::root(true) . '/index.php?option=com_vmcustomadvanced&format=json&task=field.getChildVariants';

        $field->display .= PlgVmCustomAdvancedHelper::renderLayout('default', ['product' => $product, 'field' => $field, 'fields' => $fields, 'variants' => $fields->variants, 'max_level' => $max_level, 'query_url' => $query_url]);

        $this->processedFields[$hash] = $field->virtuemart_customfield_id;

        return true;
    }

    public function onDisplayProductSubVariants($subject, $product, $field, &$html)
    {
        if ($field->custom_element != $this->_name) {
            return;
        }

        if (!($fields = $this->getFullFieldsData($product, $field))) {
            return;
        }

        $selected  = $this->app->input->get('selected', [], 'ARRAY');
        $level     = $this->app->input->getInt('level', 0);
        $max_level = count($fields->sets);

        if (empty($selected)) {
            $subject->setErrorMessage('Nothing selected.');
            return false;
        }

        if ($level <= 0) {
            $subject->setErrorMessage('Invalid level.');
            return false;
        }

        if ($level > $max_level) {
            // No child exists
            return true;
        }

        asort($selected);

        $childFields = $this->findChildField($fields, $selected, $level);

        if (empty($childFields)) {
            $subject->setErrorMessage('No child fields available.');

            return false;
        }

        $childFields->virtuemart_custom_id = $fields->virtuemart_custom_id;
        $childFields->default_selected     = $fields->default_selected;
        $childFields->required             = $fields->required;
        $childFields->multiple             = $fields->multiple;
        $childFields->show_price           = $fields->show_price;
        $childFields->stock_check          = $fields->stock_check;

        $html .= PlgVmCustomAdvancedHelper::renderLayout('variants', ['product' => $product, 'field' => $field, 'fields' => $childFields, 'variants' => $childFields->variants, 'max_level' => $max_level]);

        return true;
    }

    public function plgVmOnCheckoutCheckStock(&$cart, &$product, &$quantity, &$errorMsg, &$adjustQ)
    {
        if (!empty($product->customfields)) {
            foreach ($product->customfields as $k => $productCustom) {
                $selected = -1;

                if (isset($product->cart_item_id)) {
                    if (isset($cart->cartProductsData[$product->cart_item_id]['customProductData'][$productCustom->virtuemart_custom_id]) && !is_array($cart->cartProductsData[$product->cart_item_id]['customProductData'][$productCustom->virtuemart_custom_id]) && $cart->cartProductsData[$product->cart_item_id]['customProductData'][$productCustom->virtuemart_custom_id] == $productCustom->virtuemart_customfield_id) {
                        $selected = $productCustom->virtuemart_customfield_id;
                    } elseif (isset($cart->cartProductsData[$product->cart_item_id]['customProductData'][$productCustom->virtuemart_custom_id]) && is_array($cart->cartProductsData[$product->cart_item_id]['customProductData'][$productCustom->virtuemart_custom_id]) && isset($cart->cartProductsData[$product->cart_item_id]['customProductData'][$productCustom->virtuemart_custom_id][$productCustom->virtuemart_customfield_id])) {
                        $selected = $cart->cartProductsData[$product->cart_item_id]['customProductData'][$productCustom->virtuemart_custom_id][$productCustom->virtuemart_customfield_id];
                    }
                } else {
                    $pluginFields = vRequest::getVar('customProductData', null);

                    if ($pluginFields == null && isset($product->customPlugin)) {
                        $pluginFields = json_decode($product->customPlugin, true);
                    }

                    if (isset($pluginFields[$product->virtuemart_product_id][$productCustom->virtuemart_custom_id]) && !is_array($pluginFields[$product->virtuemart_product_id][$productCustom->virtuemart_custom_id]) && $pluginFields[$product->virtuemart_product_id][$productCustom->virtuemart_custom_id] == $productCustom->virtuemart_customfield_id) {
                        $selected = $productCustom->virtuemart_customfield_id;
                    } elseif (isset($pluginFields[$product->virtuemart_product_id][$productCustom->virtuemart_custom_id]) && is_array($pluginFields[$product->virtuemart_product_id][$productCustom->virtuemart_custom_id]) && isset($pluginFields[$product->virtuemart_product_id][$productCustom->virtuemart_custom_id][$productCustom->virtuemart_customfield_id])) {
                        $selected = $pluginFields[$product->virtuemart_product_id][$productCustom->virtuemart_custom_id][$productCustom->virtuemart_customfield_id];
                    }
                }

                if ($selected === -1) {
                    continue;
                }

                if (!empty($productCustom) && $productCustom->field_type == 'E' && $productCustom->custom_element == $this->_name && $productCustom->stock_check) {
                    $inStock = $productCustom->stock - $productCustom->ordered;
                    $existingQuantity = 0;

                    if (!empty($cart->cartProductsData)) {
                        foreach ($cart->cartProductsData as $cart_item_id => $item) {
                            if (isset($product->cart_item_id) && $product->cart_item_id == $cart_item_id) {
                                continue;
                            }

                            if (!empty($item['customProductData']) && isset($item['customProductData'][$productCustom->virtuemart_custom_id]) && is_array($item['customProductData'][$productCustom->virtuemart_custom_id]) && isset($item['customProductData'][$productCustom->virtuemart_custom_id][$productCustom->virtuemart_customfield_id]) && $item['customProductData'][$productCustom->virtuemart_custom_id][$productCustom->virtuemart_customfield_id] == 1) {
                                $existingQuantity += $item['quantity'];
                            }
                        }
                    }

                    $usableStock = $inStock - $existingQuantity;

                    if ($usableStock < $quantity) {
                        $sets = $this->getSets($productCustom->field_sets);

                        if (empty($sets)) {
                            return true;
                        }

                        $selectedOptions = (array) $productCustom->sets;
                        $selectVariantName = [];
                        $lang = Factory::getLanguage();

                        foreach ($sets as $set) {
                            if (isset($selectedOptions[$set->id])) {
                                $selectOptionID = (int) $selectedOptions[$set->id];

                                if (isset($set->options[$selectOptionID])) {
                                    $selectedOption = $set->options[$selectOptionID];

                                    $selectVariantName[] = ($lang->hasKey($set->alias) ? JText::_($set->alias) : $set->title) . ': ' . ($lang->hasKey($selectedOption->alias) ? JText::_($selectedOption->alias) : $selectedOption->title);
                                }
                            }
                        }

                        $selectVariantName = implode('; ', $selectVariantName);

                        $product->errorMsg = JText::sprintf('COM_VMCUSTOMADVANCED_ERROR_OUT_OF_STOCK', $product->product_name, $selectVariantName, $inStock);

                        vmWarn($product->errorMsg);

                        $quantity = $usableStock > 0 ? $usableStock : 0;
                        $adjustQ = true;

                        return false;
                    }
                }
            }
        }
    }

    public function plgVmGetProductStockToUpdateByCustom(&$table, $param, $field)
    {
        if (empty($field->custom_element) || $field->custom_element != $this->_name) {
            return;
        }

        if (is_array($param)) {
            foreach ($param as $virtuemart_customfield_id => $value) {
                if ($virtuemart_customfield_id > 0 && $value == 1) {
                    $customField = $this->getCustomEmbeddedProductCustomField($virtuemart_customfield_id);

                    if ($customField) {
                        $this->updateStock($table, $customField);
                    }
                }
            }
        } else {
            $this->updateStock($table, $field);
        }
    }

    protected function getCustomEmbeddedProductCustomField($virtuemart_customfield_id)
    {
        $db    = Factory::getDBO();
        $query = VirtueMartModelCustomfields::getProductCustomSelectFieldList();

        if ($virtuemart_customfield_id) {
            $query .= ' WHERE ' . $db->quoteName('virtuemart_customfield_id') . ' = ' . (int) $virtuemart_customfield_id;
        }

        $db->setQuery($query);
        $field = $db->loadObject();

        if ($field) {
            VirtueMartModelCustomfields::bindCustomEmbeddedFieldParams($field, $field->field_type);
        }

        return $field;
    }

    protected function updateStock(&$table, &$field)
    {
        JTable::addIncludePath(JPATH_ADMINISTRATOR . '/components/com_vmcustomadvanced/tables');

        $oTable = JTable::getInstance('order_items', 'VMCustomAdvancedTable');

        $isNew = false;
        $data  = [];

        $data['virtuemart_order_item_id']  = $table->virtuemart_order_item_id;
        $data['virtuemart_customfield_id'] = $field->virtuemart_customfield_id;
        $data['order_status']              = $table->order_status;
        $data['product_quantity']          = $table->product_quantity;

        if (!$oTable->load(['virtuemart_order_item_id' => $table->virtuemart_order_item_id, 'virtuemart_customfield_id' => $field->virtuemart_customfield_id])) {
            $isNew = true;
        }

        if ($oTable->id && $oTable->order_status == $table->order_status && $oTable->product_quantity == $table->product_quantity) {
            // Nothing to be done
            return;
        }

        $orderStates = $this->getOrderStates();
        $orderStates['N'] = ['order_status_id' => 0 , 'order_status_code' => 'N' , 'order_stock_handle' => 'A'];

        if (($oTable->order_status && !array_key_exists($oTable->order_status, $orderStates)) || !array_key_exists($table->order_status, $orderStates)) {
            // Unknown order status. Can't process further.
            return ;
        }

        // P  Pending
        // C  Confirmed
        // X  Cancelled
        // R  Refunded
        // S  Shipped
        // N  New or coming from cart

        // To have no product setted as ordered when added to cart simply delete 'P' FROM array Reserved
        // don't set same values in the 2 arrays !!!
        // stockOut is in normal case shipped product

        // order_stock_handle
        // 'A' : Stock Available
        // 'O' : Stock Out
        // 'R' : Stock reserved
        // the status decreasing real stock ?
        // $stockOut = array('S');

        $isOut         = false;
        $wasOut        = false;
        $isReserved    = false;
        $wasReserved   = false;
        $oldState      = $oTable->order_status;
        $newState      = $table->order_status;
        $quantity      = (int) ($oTable->product_quantity ? $oTable->product_quantity : $table->product_quantity);
        $stockAction   = null;
        $orderedAction = null;
        $difference    = $table->product_quantity - $quantity;

        if ($orderStates[$newState]['order_stock_handle'] == 'O') {
            $isOut = true;
        }

        if ($oldState && $orderStates[$oldState]['order_stock_handle'] == 'O') {
            $wasOut = true;
        }

        if ($orderStates[$newState]['order_stock_handle'] == 'R') {
            $isReserved = true;
        }

        if ($oldState && $orderStates[$oldState]['order_stock_handle'] == 'R') {
            $wasReserved = true;
        }

        if ($isOut && !$wasOut) {
            $stockAction = '-';
        } elseif ($wasOut && !$isOut) {
            $stockAction = '+';
        }

        if ($isReserved && !$wasReserved) {
            $orderedAction = '+';
        } elseif (!$isReserved && $wasReserved) {
            $orderedAction = '-';
        }

        if (!empty($difference) && !$stockAction && !$orderedAction) {
            if ($isReserved) {
                if ($difference > 0) {
                    $orderedAction = '+';
                } else {
                    $orderedAction = '-';
                }
            }

            if ($isOut) {
                if ($difference > 0) {
                    $stockAction = '+';
                } else {
                    $orderedAction = '-';
                }
            }

            $quantity = abs($difference);
        }

        $cData = [];

        $cData['field_type']                = $field->field_type;
        $cData['custom_element']            = $field->custom_element;
        $cData['custom_jplugin_id']         = $field->custom_jplugin_id;
        $cData['virtuemart_custom_id']      = $field->virtuemart_custom_id;
        $cData['virtuemart_product_id']     = $field->virtuemart_product_id;
        $cData['virtuemart_customfield_id'] = $field->virtuemart_customfield_id;
        $cData['ordering']                  = $field->ordering;
        $cData['customfield_value']         = $field->customfield_value;
        $cData['customfield_price']         = $field->customfield_price;
        $cData['override']                  = $field->override;
        $cData['disabler']                  = $field->disabler;
        $cData['sets']                      = (array) $field->sets;
        $cData['stock']                     = (int) $field->stock;
        $cData['ordered']                   = (int) $field->ordered;

        if ($stockAction) {
            if ($stockAction == '-') {
                $cData['stock'] -= $quantity;
            } else {
                $cData['stock'] += $quantity;
            }
        }

        if ($orderedAction) {
            if ($orderedAction == '-') {
                $cData['ordered'] -= $quantity;
            } else {
                $cData['ordered'] += $quantity;
            }
        }

        $field->stock = $cData['stock'];
        $field->ordered = $cData['ordered'];

        if ($stockAction || $orderedAction) {
            $customfieldsTable = VmTable::getInstance('product_customfields', 'Table', ['dbo' => Factory::getDbo()]);

            //$customfieldsTable->setPrimaryKey('virtuemart_product_id'); We do not need this in VirtueMart 4.
            $customfieldsTable->_xParams = 'customfield_params';

            VirtueMartModelCustom::setParameterableByFieldType($customfieldsTable, $cData['field_type'], $cData['custom_element'], $cData['custom_jplugin_id']);

            $params = [];

            foreach ($customfieldsTable->_varsToPushParam as $name => $attrib) {
                if (isset($cData[$name])) {
                    $params[$name] = $attrib;
                } else {
                    unset($customfieldsTable->$name);
                }
            }

            $customfieldsTable->_varsToPushParam = $params;

            if (!$customfieldsTable->bindChecknStore($cData)) {
                // Stock could not be updated.
                $this->app->enqueueMessage(__CLASS__ . '::' . __METHOD__ . '() Failed to update stock.', 'error');
                return false;
            }
        }

        if (!$oTable->bind($data)) {
            $this->app->enqueueMessage($table->getError(), 'error');
            return false;
        }

        if (!$oTable->check()) {
            $this->app->enqueueMessage($table->getError(), 'error');
            return false;
        }

        if (!$oTable->store()) {
            $this->app->enqueueMessage($table->getError(), 'error');
            return false;
        }

        return true;
    }

    public function plgVmOnViewCartVM3(&$product, &$field, &$html)
    {
        if (empty($field->custom_element) || $field->custom_element != $this->_name) {
            return;
        }

        if (empty($product->customProductData[$field->virtuemart_custom_id])) {
            return;
        }

        $variantmods    = isset($product->product_attribute) ? $product->product_attribute : null;
        $variantmods    = isset($product->customProductData) ? $product->customProductData : $variantmods;
        $selectedFields = [];
        $selected       = [];

        if ($variantmods && !is_array($variantmods)) {
            $variantmods = json_decode($variantmods, true);
        }

        // Probably VirtueMart system has changed. We show all results by default.
        if ($variantmods == null || (isset($variantmods[$field->virtuemart_custom_id]) && !is_array($variantmods[$field->virtuemart_custom_id]) && $variantmods[$field->virtuemart_custom_id] == $field->virtuemart_customfield_id)) {
            $selectedFields[] = (int) $field->virtuemart_customfield_id;
        } elseif (isset($variantmods[$field->virtuemart_custom_id]) && isset($variantmods[$field->virtuemart_custom_id][$field->virtuemart_customfield_id]) && $variantmods[$field->virtuemart_custom_id][$field->virtuemart_customfield_id] == 1) {
            foreach ($variantmods[$field->virtuemart_custom_id] as $virtuemart_customfield_id => $value) {
                if ($value == 1) {
                    $selectedFields[] = (int) $virtuemart_customfield_id;
                }
            }
        }

        if (empty($selectedFields)) {
            return;
        }

        $hash = md5('virtuemart_product_id:' . $field->virtuemart_product_id . '.virtuemart_custom_id:' . $field->virtuemart_custom_id . '.variantmods:' . serialize($variantmods) . '.cartLayout:' . $this->cartLayout);

        if (isset($this->cartSelectedFields[$hash]) && $this->cartSelectedFields[$hash] != $field->virtuemart_customfield_id) {
            return;
        }

        //We let that here as Fallback
        if (empty($product->customfields)) {
            $productDB = VmModel::getModel('product')->getProduct($product->virtuemart_product_id);

            if ($productDB && !empty($productDB->customfields)) {
                $product->customfields = $productDB->customfields;
            } else {
                $product->customfields = array();
            }
        }

        foreach ($product->customfields as $customField) {
            if (!is_object($customField) || $customField->virtuemart_custom_id != $field->virtuemart_custom_id) {
                continue;
            }

            if (!in_array($customField->virtuemart_customfield_id, $selectedFields)) {
                continue;
            }

            $temp = (array) $customField->sets;

            foreach ($temp as $set_id => $option_id) {
                if (!isset($selected[$set_id])) {
                    $selected[$set_id] = [];
                }

                if (!in_array($option_id, $selected[$set_id])) {
                    $selected[$set_id][] = $option_id;
                }
            }
        }

        $sets = $this->getSets($field->field_sets);

        if (empty($sets)) {
            return;
        }

        $html .= PlgVmCustomAdvancedHelper::renderLayout($this->cartLayout, ['product' => $product, 'field' => $field, 'sets' => $sets, 'selected' => $selected]);

        $this->cartSelectedFields[$hash] = $field->virtuemart_customfield_id;

        return true;
    }

    public function plgVmOnViewCartModuleVM3(&$product, &$field, &$html)
    {
        return $this->plgVmOnViewCartVM3($product, $field, $html);
    }

    public function plgVmDisplayInOrderFEVM3(&$product, &$field, &$html)
    {
        if (empty($field->custom_element) || $field->custom_element != $this->_name) {
            return;
        }

        if (empty($product->customProductData[$field->virtuemart_custom_id])) {
            return;
        }

        $input  = $this->app->input;
        $option = strtolower($input->getCmd('option', ''));
        $view   = strtolower($input->getCmd('view', ''));
        $task   = strtolower($input->getCmd('task', ''));

        if ($this->app->isClient('site') && $option == 'com_virtuemart' && $view == 'orders' && $task != 'callinvoiceview') {
            $this->cartLayout = 'order';
        } else {
            // PDF invoice and emails can not render colored buttons and images. So we use a simpler layout with all text information.
            $this->cartLayout = 'invoice';
        }

        $this->plgVmOnViewCartVM3($product, $field, $html);

        $this->cartLayout = 'cart';
    }

    public function plgVmDisplayInOrderBEVM3(&$product, &$field, &$html)
    {
        $this->plgVmDisplayInOrderFEVM3($product, $field, $html);
    }

    public function plgVmDisplayInOrderBE(&$item, $productCustom, &$html)
    {
        if (!empty($productCustom)) {
            $item->productCustom = $productCustom;
        }

        if (empty($item->productCustom->custom_element) || $item->productCustom->custom_element != $this->_name) {
            return;
        }

        $this->plgVmDisplayInOrderFEVM3($item, $productCustom, $html);
    }

    public function plgVmOnStoreInstallPluginTable($psType, $data, $table)
    {
        if ($psType != $this->_psType) {
            return;
        }

        if (empty($table->custom_element) || $table->custom_element != $this->_name) {
            return;
        }

        if (empty($table->is_input) || empty($table->is_cart_attribute) || $table->layout_pos != 'addtocart') {
            $table->is_input = 1;
            $table->is_cart_attribute = 1;
            $table->layout_pos = 'addtocart';

            $table->store();
        }

        //Should we use an own internal variable or store it in the params?

        //Here is no getVmPluginCreateTableSQL defined
        //return $this->onStoreInstallPluginTable($psType);
    }

    public function plgVmDeclarePluginParamsCustomVM3(&$data)
    {
        return $this->declarePluginParams('custom', $data);
    }

    public function plgVmGetTablePluginParams($psType, $name, $id, &$xParams, &$varsToPush)
    {
        return $this->getTablePluginParams($psType, $name, $id, $xParams, $varsToPush);
    }

    public function plgVmSetOnTablePluginParamsCustom($name, $id, &$table, $xParams)
    {
        return $this->setOnTablePluginParams($name, $id, $table, $xParams);
    }

    public function plgVmOnDisplayEdit($virtuemart_custom_id, &$customPlugin)
    {
        return $this->onDisplayEditBECustom($virtuemart_custom_id, $customPlugin);
    }

    public function plgVmPrepareCartProduct(&$product, &$customfield, $selected, &$modificatorSum)
    {
        if ($customfield->custom_element != $this->_name) {
            return;
        }

        if (!empty($selected) && $selected != -1) {
            $modificatorSum += $customfield->customfield_price ;
        } else {
            $modificatorSum += 0.0;
        }

        return true;
    }

    public function plgVmOnSelfCallFE($type, $name, &$render)
    {
        $render->html = '';
    }

    public function plgVmBeforeStoreProduct(&$data, &$productTable)
    {
        if (!empty($data['field'])) {
            $filteringOptions = [];

            foreach ($data['field'] as $key => $field) {
                if ($field['custom_element'] != $this->_name) {
                    continue;
                }

                if (!empty($data['customfield_params'][$key]) && !isset($data['clone'])) {
                    if (array_key_exists($key, $data['customfield_params'])) {
                        $field = array_merge((array) $field, (array) $data['customfield_params'][$key]);
                    }
                }

                if (!empty($field['sets']) && is_array($field['sets'])) {
                    $nonEmptySets = array_filter($field['sets']);

                    if (count($field['sets']) > count($nonEmptySets)) {
                        // No option selected for some of its sets. So removed it.
                        unset($data['field'][$key]);
                    } else {
                        foreach ($field['sets'] as $set_id => $option_id) {
                            $set = $this->getSet($set_id);
                            $option_id = (int) $option_id;

                            if (empty($set) || !$set->filter_support || !$set->virtuemart_custom_id || !isset($set->options[$option_id])) {
                                continue;
                            }

                            if (!self::customExists($set->virtuemart_custom_id)) {
                                continue;
                            }

                            if (!isset($filteringOptions[$set->virtuemart_custom_id])) {
                                $filteringOptions[$set->virtuemart_custom_id] = [];
                            }

                            $option = $set->options[$option_id];

                            $filteringOptions[$set->virtuemart_custom_id][$option->id] = [
                                'field_type'                => 'S',
                                'custom_element'            => 0,
                                'custom_jplugin_id'         => 0,
                                'virtuemart_custom_id'      => $set->virtuemart_custom_id,
                                'virtuemart_product_id'     => $field['virtuemart_product_id'],
                                'virtuemart_customfield_id' => 0,
                                'ordering'                  => 0,
                                'customfield_price'         => '0.000000',
                                'customfield_value'         => $set->translatable_filter_options ? $option->alias : $option->title,
                                'disabler'                  => !empty($field['disabler']) ? $field['disabler'] : 0,
                                'override'                  => !empty($field['override']) ? $field['override'] : 0
                            ];
                        }
                    }
                }
            }

            $ordering = 0;

            // Check and remove all existing filter fields
            foreach ($data['field'] as $key => $field) {
                $virtuemart_custom_id = (int) $field['virtuemart_custom_id'];

                if (array_key_exists($virtuemart_custom_id, $filteringOptions)) {
                    unset($data['field'][$key]);
                }

                $ordering = $field['ordering'];
            }

            // Add all filter fields
            foreach ($filteringOptions as $virtuemart_custom_id => $options) {
                foreach ($options as $option_id => $field) {
                    $ordering++;

                    $field['ordering'] = $ordering;

                    $data['field'][] = $field;
                }
            }
        }
    }

    private function getOptionVariables()
    {
        $vars = [
            'field_sets'       => [[], 'array'],
            'default_selected' => [1, 'int'],
            'required'         => [0, 'int'],
            'stock_check'      => [0, 'int'],
            'show_price'       => [1, 'int'],
            'multiple'         => [0, 'int'],
            'stock'            => [0, 'int'],
            'ordered'          => [0, 'int'],
            'sets'             => [[], 'array'],
        ];

        return $vars;
    }

    private function getSets($set_ids)
    {
        $sets = [];

        foreach ($set_ids as $set_id) {
            if (!empty($set_id)) {
                $set = $this->getSet($set_id);

                if (!empty($set)) {
                    $sets[] = $set;
                }
            }
        }

        // Sort by actual set ordering
        usort($sets, function ($a, $b) {
            return $a->ordering - $b->ordering;
        });

        $temp = [];
        $level = 1;

        foreach ($sets as $set) {
            $set->level = $level;
            $set->availableOptions = [];
            $set->parentChildren = [];

            $level++;
        }

        return $sets;
    }

    private function getSet($set_id)
    {
        $set_id = (int) $set_id;

        if ($set_id <= 0) {
            return false;
        }

        if (!isset(self::$sets[$set_id])) {
            $db    = Factory::getDbo();
            $query = $db->getQuery(true);

            $query->select('*')
                ->from($db->quoteName('#__vmcustomadvanced_sets'))
                ->where($db->quoteName('state') . ' = 1')
                ->where($db->quoteName('id') . ' = ' . $set_id);

            $db->setQuery($query);
            $set = $db->loadObject();

            if (!empty($set)) {
                $set->params = new Registry($set->params);

                $query->clear()
                    ->select('*')
                    ->from($db->quoteName('#__vmcustomadvanced_options'))
                    ->where($db->quoteName('state') . ' = 1')
                    ->where($db->quoteName('set_id') . ' = ' . $set->id);

                $db->setQuery($query);
                $options = $db->loadObjectList('id');

                $set->options = !empty($options) ? $options : [];

                $set->listOptions = [];

                $set->listOptions[] = HTMLHelper::_('select.option', '', JText::_('COM_VMCUSTOMADVANCED_SELECT_AN_OPTION'));

                foreach ($set->options as $option) {
                    $option->params = new Registry($option->params);

                    $set->listOptions[] = HTMLHelper::_('select.option', $option->id, $option->title);
                }

                self::$sets[$set_id] = $set;
            } else {
                self::$sets[$set_id] = false;
            }
        }

        return self::$sets[$set_id];
    }

    private function getFullFieldsData($product, $field)
    {
        if (empty($product->customfields)) {
            return false;
        }

        $sets = $this->getSets($field->field_sets);

        if (empty($sets)) {
            return false;
        }

        // Reset cache
        self::$data = null;

        $data = new stdClass();
        $data->virtuemart_custom_id = 0;
        $data->sets = $sets;
        $data->variants = new stdClass();
        $data->variants->level = null;
        $data->variants->set = null;
        $data->variants->options = null;

        foreach ($product->customfields as &$customfield) {
            if ($customfield->virtuemart_custom_id != $field->virtuemart_custom_id || empty($customfield->sets)) {
                continue;
            }

            if (empty($data->virtuemart_custom_id)) {
                $data->virtuemart_custom_id = $field->virtuemart_custom_id;
                $data->default_selected = $customfield->default_selected;
                $data->required = $customfield->required;
                $data->multiple = $customfield->multiple;
                $data->show_price = $customfield->show_price;
                $data->stock_check = $customfield->stock_check;
            }

            $customfield_id = (int) $customfield->virtuemart_customfield_id;

            foreach ($sets as $key => $set) {
                if (!isset($customfield->sets->{$set->id}) || empty($customfield->sets->{$set->id})) {
                    continue;
                }

                $level = $set->level;
                $set_id = (int) $set->id;
                $option_id = (int) $customfield->sets->$set_id;

                if (!isset($set->options[$option_id])) {
                    continue;
                }

                if ($level == 1) {
                    $data->variants->set = $set;
                    $data->variants->level = $level;

                    if (!isset($data->variants->options[$option_id])) {
                        $data->variants->options[$option_id] = clone $set->options[$option_id];
                    }

                    if (!property_exists($data->variants->options[$option_id], 'variants')) {
                        $data->variants->options[$option_id]->variants = null;
                    }

                    /* If single level */
                    if (count($sets) == 1) {
                        $data->variants->options[$option_id]->stock = $customfield->stock;
                        $data->variants->options[$option_id]->ordered = $customfield->ordered;
                        $data->variants->options[$option_id]->price = $customfield->customfield_price;
                        $data->variants->options[$option_id]->virtuemart_customfield_id = $customfield_id;
                    } else {
                        $data->variants->options[$option_id]->stock = null;
                        $data->variants->options[$option_id]->ordered = null;
                        $data->variants->options[$option_id]->price = null;
                        $data->variants->options[$option_id]->virtuemart_customfield_id = 0;
                    }
                } elseif ($level > 1) {
                    $parentKey = $key - 1;
                    $childKey = $key + 1;

                    if (isset($sets[$parentKey])) {
                        $parentSet = $sets[$parentKey];
                        $parent_set_id = $parentSet->id;

                        if (!empty($customfield->sets->$parent_set_id)) {
                            $parent_option_id = $customfield->sets->$parent_set_id;

                            if (isset($parentSet->options[$parent_option_id])) {
                                $this->addChildVariant($data, $sets, $parent_option_id, $option_id, $customfield_id, $set, $parentKey, $childKey, $level, $customfield);
                            }
                        }
                    }
                }
            }
        }

        return $data;
    }

    private function addChildVariant(&$data, $sets, $parent_option_id, $option_id, $customfield_id, $set, $parentKey, $childKey, $level, $customfield)
    {
        $hasChild = false;

        if (isset($sets[$childKey])) {
            $child_set_id = $sets[$childKey]->id;

            if (!empty($customfield->sets->$child_set_id)) {
                $hasChild = true;
            }
        }

        if (isset($data->variants->options[$parent_option_id]) && !empty($data->variants->options[$parent_option_id]->id)) {
            if (!$data->variants->options[$parent_option_id]->variants) {
                $data->variants->options[$parent_option_id]->variants = new stdClass();
                $data->variants->options[$parent_option_id]->variants->level = $level;
                $data->variants->options[$parent_option_id]->variants->set = $set;
                $data->variants->options[$parent_option_id]->variants->options = [];
            }

            if (isset($data->variants->options[$parent_option_id]->variants->options[$option_id])) {
                $data->variants->options[$parent_option_id]->variants->options[$option_id]->customfield_ids[] = $customfield_id;
            } else {
                $option = clone $set->options[$option_id];
                $option->variants = null;

                if (!$hasChild) {
                    $option->stock = $customfield->stock;
                    $option->ordered = $customfield->ordered;
                    $option->price = $customfield->customfield_price;
                    $option->virtuemart_customfield_id = $customfield->virtuemart_customfield_id;
                } else {
                    $option->stock = null;
                    $option->ordered = null;
                    $option->price = null;
                    $option->virtuemart_customfield_id = 0;
                }

                $option->customfield_ids = [$customfield_id];

                $data->variants->options[$parent_option_id]->variants->options[$option_id] = $option;
            }

            // Save the last parent data in cache with inheritance
            self::$data = &$data;
        } else {
            $parentParentKey = $parentKey - 1;

            if (isset($sets[$parentParentKey])) {
                $parentParentSet = $sets[$parentParentKey];
                $parent_parent_set_id = $parentParentSet->id;

                if (!empty($customfield->sets->$parent_parent_set_id)) {
                    $parent_parent_option_id = $customfield->sets->$parent_parent_set_id;

                    if (isset($parentParentSet->options[$parent_parent_option_id])) {
                        if (self::$data && isset(self::$data->variants->options[$parent_parent_option_id]) && !empty(self::$data->variants->options[$parent_parent_option_id]->id)) {
                            $this->addChildVariant(self::$data->variants->options[$parent_parent_option_id], $sets, $parent_option_id, $option_id, $customfield_id, $set, $parentParentKey, $childKey, $level, $customfield);
                        }
                    }
                }
            }
        }
    }

    private function findChildField($fields, $selected, $level)
    {
        foreach ($selected as $levelKey => $item) {
            if ($fields->variants->level = $levelKey && $fields->variants->set->id == $item['set_id']) {
                foreach ($fields->variants->options as $option) {
                    if ($option->id == $item['option_id']) {
                        unset($selected[$levelKey]);

                        if (!empty($option->variants) && !empty($option->variants->options)) {
                            if ($option->variants->level == $level) {
                                return $option;
                            } elseif ($option->variants->level < $level) {
                                return $this->findChildField($option, $selected, $level);
                            }
                        }
                    }
                }
            }
        }
    }

    private static function customExists($virtuemart_custom_id)
    {
        static $cache = [];

        $virtuemart_custom_id = (int) $virtuemart_custom_id;

        if (!isset($cache[$virtuemart_custom_id])) {
            $db = Factory::getDbo();
            $query = $db->getQuery(true);

            $query->select($db->quoteName('field_type'))
                ->from($db->quoteName('#__virtuemart_customs'))
                ->where($db->quoteName('virtuemart_custom_id') . ' = ' . $virtuemart_custom_id);

            $db->setQuery($query);
            $field_type = $db->loadResult();

            $cache[$virtuemart_custom_id] = $field_type == 'S';
        }

        return $cache[$virtuemart_custom_id];
    }

    private function getOrderStates()
    {
        static $statuses = null;

        if ($statuses === null) {
            $db    = Factory::getDbo();
            $query = $db->getQuery(true);

            $query->select('*')
                ->from($db->quoteName('#__virtuemart_orderstates'));

            $db->setQuery($query);
            $statuses = $db->loadAssocList('order_status_code');
        }

        return $statuses;
    }

    private function isHTML()
    {
        $document = Factory::getDocument();
        $input    = $this->app->input;
        $isFeed   = ($document->getType() == 'feed' || $input->getWord('format') == 'feed' || $input->getWord('type') == 'rss' || $input->getWord('type') == 'atom');
        $isHTML   = ($document->getType() == 'html' && !$isFeed);

        return $isHTML;
    }
}
