<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

// phpcs:disable PSR1.Files.SideEffects
defined('_JEXEC') or die;

include_once JPATH_SITE . '/components/com_vmcustomadvanced/define.php';
// phpcs:enable PSR1.Files.SideEffects

class PlgVmCustomAdvancedHelper
{
    public static function renderLayout($layout, $displayData = [])
    {
        $layout_path = JPluginHelper::getLayoutPath('vmcustom', 'advanced', $layout);

        if (!is_file($layout_path)) {
            throw new RuntimeException(JText::sprintf('PLG_VMCUSTOM_ADVANCED_ERROR_LAYOUT_NOT_FOUND', $layout_path), 500);
        }

        $defaultData = [
            'version' => self::getVersion()
        ];

        $displayData = array_merge($defaultData, $displayData);

        // Render the layout
        ob_start();
        include $layout_path;
        $html = ob_get_clean();

        return $html;
    }

    public static function sortArrayByArray(array $array, array $orderArray)
    {
        $ordered = [];

        foreach ($orderArray as $key) {
            if (array_key_exists($key, $array)) {
                $ordered[$key] = $array[$key];
                unset($array[$key]);
            }
        }

        return $ordered + $array;
    }

    public static function getVersion()
    {
        static $version = null;

        if ($version === null) {
            $manifest = __DIR__ . '/advanced.xml';

            if (empty($manifest)) {
                $version = 'unknown';
            } else {
                $xml     = simplexml_load_file($manifest);
                $version = (string) $xml->version;
            }
        }

        return $version;
    }

    public static function sort(&$array)
    {
        return uasort($array, ['self', 'compareOrdering']);
    }

    public static function compareOrdering($object1, $object2)
    {
        return $object1->ordering > $object2->ordering;
    }
}
