<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\CMS\Helper\MediaHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;

defined('_JEXEC') or die;

extract($displayData);

HTMLHelper::_('stylesheet', 'com_vmcustomadvanced/style.css', ['version' => 'ver=' . $version, 'relative' => true]);

$lang = Factory::getLanguage();
$html = [];

foreach ($sets as $set) {
    if (!empty($selected[$set->id])) {
        $optionHtmls = [];

        foreach ($selected[$set->id] as $option_id) {
            if (!isset($set->options[$option_id])) {
                continue;
            }

            $option = $set->options[$option_id];
            $title  = $lang->hasKey($option->alias) ? Text::_($option->alias) : $option->title;
            $sample = '';
            $optionHtml = [];

            if (!empty($option->image) && version_compare(JVERSION, '4.0.0', 'ge')) {
                $option->image = MediaHelper::getCleanMediaFieldValue($option->image);
            }

            if ($set->option_type == 1) {
                $sample = '<span class="plg-vmcustom-advanced-sample" style="background-color: ' . $option->color . '" title="' . htmlspecialchars($title, ENT_COMPAT, 'UTF-8') . '"></span>';
            } elseif ($set->option_type == 2) {
                if (!empty($option->image) && is_file(JPATH_SITE . '/' . $option->image)) {
                    $img = '<img src="' . htmlspecialchars(JUri::root() . $option->image, ENT_COMPAT, 'UTF-8') . '" alt="' . htmlspecialchars($title, ENT_COMPAT, 'UTF-8') . '" title="' . htmlspecialchars($title, ENT_COMPAT, 'UTF-8') . '" />';
                } else {
                    $img = '<span class="vmcustomadvanced-dummy-img" title="' . htmlspecialchars($title, ENT_COMPAT, 'UTF-8') . '"></span>';
                }

                $sample = '<span class="plg-vmcustom-advanced-sample-img">' . $img . '</span>';
            }

            $optionHtml[] = '<span class="plg-vmcustom-advanced-set-value">';

            $title  = '<span class="plg-vmcustom-advanced-option-title">' . $title . '</span>';

            if (!empty($sample)) {
                $optionHtml[] = $sample;

                if (!$set->is_button) {
                    $optionHtml[] = $title;
                }
            } else {
                $optionHtml[] = $title;
            }

            $optionHtml[] = '</span>';

            $optionHtmls[] = implode('', $optionHtml);
        }

        if (!empty($optionHtml)) {
            $html[] = '<div class="plg-vmcustom-advanced-set">';

            $html[] = '<span class="plg-vmcustom-advanced-set-title">';
            $html[] = $lang->hasKey($set->alias) ? Text::_($set->alias) : $set->title;
            $html[] = ': &nbsp;</span>';

            $html[] = implode(', ', $optionHtmls);

            $html[] = '</div>';
        }
    }
}
?>
<span class="plg-vmcustom-advanced-sets">
    <?php echo implode('', $html); ?>
</span>

