<?php

/**
 * @package    VP Advanced Custom Field for VirtueMart
 *
 * @author     Abhishek Das <info@virtueplanet.com>
 * @copyright  Copyright (C) 2012-2023 VirtuePlanet Services LLP. All rights reserved.
 * @license    GNU General Public License version 2 or later; see LICENSE.txt
 * @link       https://www.virtueplanet.com
 *
 * @phpcs:disable PSR1.Classes.ClassDeclaration.MissingNamespace
 */

use Joomla\CMS\Factory;
use Joomla\CMS\Helper\MediaHelper;
use Joomla\CMS\HTML\HTMLHelper;
use Joomla\CMS\Language\Text;

defined('_JEXEC') or die;

extract($displayData);

/** @var CurrencyDisplay $currencyDisplay */
$currencyDisplay = CurrencyDisplay::getInstance();

/** @var calculationHelper $calculator */
$calculator   = calculationHelper::getInstance();
$lang         = Factory::getLanguage();
$isFinalLevel = $variants->level == $max_level;

if (method_exists($calculator, 'setProduct')) {
    $calculator->setProduct($product);
}

if ($isFinalLevel) {
    if ($field->multiple) {
        $name  = 'customMultiProductData[' . $product->virtuemart_product_id . '][' . $field->virtuemart_custom_id . ']';
    } else {
        $name  = 'customProductData[' . $product->virtuemart_product_id . '][' . $field->virtuemart_custom_id . ']';
    }
} else {
    $name  = 'customAdvancedData[' . $product->virtuemart_product_id . '][' . $field->virtuemart_custom_id . '][' . $variants->set->id . ']';
}

$idTag    = 'customProductData-' . $product->virtuemart_product_id . '-' . $field->virtuemart_custom_id . '-' . $variants->set->id . '-' . uniqid(rand()) . '-' . $variants->level;
$options  = [];
$i        = 0;
$selected = '';
$html     = [];

if (!$fields->default_selected) {
    $options[] = HTMLHelper::_('select.option', '', Text::_('COM_VMCUSTOMADVANCED_SELECT'));
}

// Set ordering
PlgVmCustomAdvancedHelper::sort($variants->options);

foreach ($variants->options as $opt) {
    $value    = !empty($opt->virtuemart_customfield_id) ? $opt->virtuemart_customfield_id : $opt->id;
    $disabled = !empty($fields->stock_check) && $opt->stock !== null && ($opt->stock - $opt->ordered) < 1;

    if (!empty($opt->price) && method_exists($calculator, 'calculateCustomPriceWithTax')) {
        $calculator->setProduct($product, 1);

        $opt->price = $calculator->calculateCustomPriceWithTax($opt->price);
    }

    $pricePrefix = !empty($opt->price) && $opt->price > 0 ? '+' : '';
    $price       = !empty((float) $opt->price) ? $pricePrefix . $currencyDisplay->priceDisplay($opt->price) : '';

    if (!empty($opt->image) && version_compare(JVERSION, '4.0.0', 'ge')) {
        $opt->image = MediaHelper::getCleanMediaFieldValue($opt->image);
    }

    // If drop down list
    if ($variants->set->type == 0) {
        $attr = 'data-id="' . $opt->id . '"';

        if ($variants->set->option_type == 1) {
            $attr .= ' data-color="' . $opt->color . '"';
        } elseif ($variants->set->option_type == 2) {
            if (!empty($opt->image) && is_file(JPATH_SITE . '/' . $opt->image)) {
                $attr .=  ' data-image="' . htmlspecialchars(JUri::root() . $opt->image, ENT_COMPAT, 'UTF-8') . '"';
            } else {
                $attr .=  ' data-image="NA"';
            }
        }

        if (!empty($fields->show_price) && !empty($price)) {
            $attr .= ' data-price="' . $price . '"';
        }

        if ($disabled) {
            $attr .= ' disabled="disabled"';
        }

        $options[] = HTMLHelper::_('select.option', $value, ($lang->hasKey($opt->alias) ? Text::_($opt->alias) : $opt->title), ['attr' => $attr, 'option.attr' => 'option.attr']);
    } else {
        // If radio list or color buttons
        $optIdTag = $idTag . '-' . $opt->id;
        $class    = $variants->set->params->get('button_only', 0) ? ' plg-vmcustom-advanced-button' : 'plg-vmcustom-advanced-radio';
        $class   .= $variants->set->option_type ? ' has-sample' : '';
        $class   .= $variants->set->is_button ? ' vmcustomadvanced-is-btn' : '';
        $html[]   = '<div class="' . $class . '">';
        $html[]   = '<label for="' . $optIdTag . '">';
        $sample   = '';
        $title    = $lang->hasKey($opt->alias) ? Text::_($opt->alias) : $opt->title;

        if ($variants->set->option_type == 1) {
            $sample = '<span class="vmcustomadvanced-radio-sample" style="background-color: ' . $opt->color . '; color: ' . $opt->params->get('tick_color', '#fff') . ';"></span>';
        } elseif ($variants->set->option_type == 2) {
            if (!empty($opt->image) && is_file(JPATH_SITE . '/' . $opt->image)) {
                $img = '<img src="' . htmlspecialchars(JUri::root() . $opt->image, ENT_COMPAT, 'UTF-8') . '" alt="' . htmlspecialchars($title, ENT_COMPAT, 'UTF-8') . '" />';
            } else {
                $img = '<span class="vmcustomadvanced-dummy-img"></span>';
            }

            $sample = '<span class="vmcustomadvanced-radio-sample-img" style="color: ' . $opt->params->get('tick_color', '#fff') . ';">' . $img . '</span>';
        }

        $attr = 'data-id="' . $opt->id . '"';

        if ($disabled) {
            $attr .= ' disabled="disabled"';
        }

        if ($fields->default_selected && $i === 0 && !$disabled) {
            $attr .= ' checked="checked"';
        }

        $inputType = ($isFinalLevel && $field->multiple) ? 'checkbox' : 'radio';
        $html[]    = '<input type="' . $inputType . '" class="no-vm-bind" name="' . $name . '" id="' . $optIdTag . '" value="' . $value . '"' . $attr . ' />';

        if (!empty($sample)) {
            $html[] = $sample;
        }

        $html[] = '<span class="vmcustomadvanced-radio-info">';
        $html[] = '<span class="vmcustomadvanced-radio-text"><span class="vmcustomadvanced-radio-textinner">' . $title . '</span></span>';

        if (!empty($fields->show_price) && !empty($price)) {
            $html[] = '<span class="vmcustomadvanced-radio-price">' . $price . '</span>';
        }

        if ($disabled) {
            $html[] = '<span class="vmcustomadvanced-radio-nostocktext" aria-hidden="true" tabindex="-1"><span class="vmcustomadvanced-radio-nostockinner">' . Text::_('COM_VMCUSTOMADVANCED_INFO_OUT_OF_STOCK') . '</span></span>';
        }

        $html[] = '</span>';
        $html[] = '</label>';
        $html[] = '</div>';
    }

    if (!$disabled) {
        if ($i === 0 && $fields->default_selected) {
            $selected = $value;
        }

        $i++;
    }
}

if ($variants->set->type == 0) {
    $attribs = [
        'id'             => $idTag,
        'list.attr'      => 'class="no-vm-bind"' . ($isFinalLevel && $field->multiple ? ' multiple="multiple"' : ''),
        'list.translate' => false,
        'option.key'     => 'value',
        'option.text'    => 'text',
        'list.select'    => $selected,
        'option.attr'    => 'option.attr'
    ];
}

$helpHTML = [];

if (!empty($variants->set->help_title) && !empty($variants->set->help_description)) {
    $helpID    = 'vmcustomadvanced-help-' . $product->virtuemart_product_id . '-' . $field->virtuemart_custom_id . '-' . $variants->set->id . '-' . uniqid(rand());
    $helpTitle = $lang->hasKey($variants->set->help_title) ? Text::_($variants->set->help_title) : $variants->set->help_title;

    $helpHTML[] = '<div class="vmcustomadvanced-help">';
    $helpHTML[] = '<a href="#' . $helpID . '" data-vmcamodal="true">' . $helpTitle . '</a>';
    $helpHTML[] = '</div>';

    $helpHTML[] = '<div id="' . $helpID . '" class="vmcustomadvanced-modal" aria-hidden="true" role="dialog">';
    $helpHTML[] = '<div class="vmcustomadvanced-modal-dialog">';
    $helpHTML[] = '<div class="vmcustomadvanced-modal-inner">';
    $helpHTML[] = '<div class="vmcustomadvanced-modal-header">';
    $helpHTML[] = '<button type="button" class="vmcustomadvanced-modal-close" title="Close"><span aria-hidden="true">&times;</span></button>';
    $helpHTML[] = '<h3>' . $helpTitle . '</h3>';
    $helpHTML[] = '</div>';
    $helpHTML[] = '<div class="vmcustomadvanced-modal-body">';
    $helpHTML[] = $variants->set->help_description;
    $helpHTML[] = '</div>';
    $helpHTML[] = '</div>';
    $helpHTML[] = '</div>';
    $helpHTML[] = '</div>';
} elseif (!empty($variants->set->help_title)) {
    $helpTitle  = $lang->hasKey($variants->set->help_title) ? Text::_($variants->set->help_title) : $variants->set->help_title;

    $helpHTML[] = '<div class="vmcustomadvanced-help">';
    $helpHTML[] = '<span class="vmcustomadvanced-help-text">' . $helpTitle . '</span>';
    $helpHTML[] = '</div>';
}
?>
<div class="plg-vmcustom-advanced-variant" data-level="<?php echo $variants->level; ?>" data-setid="<?php echo $variants->set->id; ?>">
    <label class="plg-vmcustom-advanced-variant-title" for="<?php echo $idTag; ?>">
        <?php echo $lang->hasKey($variants->set->alias) ? Text::_($variants->set->alias) : $variants->set->title; ?>
    </label>
    <?php if ($variants->set->type == 0) : ?>
        <?php echo HTMLHelper::_('select.genericlist', $options, $name, $attribs); ?>
    <?php else : ?>
        <?php echo implode('', $html); ?>
    <?php endif; ?>
    <?php echo implode('', $helpHTML); ?>
    <div class="vmcustomadvanced-variant-error-msg">
        <?php echo Text::_('COM_VMCUSTOMADVANCED_ERROR_SELECT'); ?>
    </div>
</div>
<?php if ($fields->default_selected && isset($variants->options[$selected]) && !empty($variants->options[$selected]->variants)) : ?>
    <?php echo PlgVmCustomAdvancedHelper::renderLayout('variants', ['product' => $product, 'field' => $field, 'fields' => $fields, 'variants' => $variants->options[$selected]->variants, 'max_level' => $max_level]); ?>
<?php endif; ?>
